unit Janela;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, OpenGL,
  ExtCtrls, OGLPanel, StdCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    OGLPanel1: TOGLPanel;
    ScrollZ: TScrollBar;
    LabelObsZ: TLabel;
    Label2: TLabel;
    ScrollX: TScrollBar;
    LabelObsX: TLabel;
    LabelObsY: TLabel;
    ScrollAngVis: TScrollBar;
    ScrollY: TScrollBar;

    procedure OGLPanel1OGLPaint;
    procedure DefineLuz;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ScrollZChange(Sender: TObject);
    procedure ScrollAngVisChange(Sender: TObject);
    procedure ScrollXChange(Sender: TObject);
    procedure ScrollYChange(Sender: TObject);

  private
    { Private declarations }
    Angulo: Real;

    AngVisao, ObsX, ObsY, ObsZ: GLFloat;


    Procedure PosicionaObservador;
    Procedure DesenhaObjeto;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}



{ **********************************************************************
               Desenha um Cubo com arestas de tamanho 20
  **********************************************************************  }
procedure TForm1.desenhaObjeto();
begin

  glBegin(GL_POLYGON);  // face posterior
    glNormal3f(0,0,-1);
    glVertex3f(10.0, 10.0, -10.0);
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(-10.0, 10.0, -10.0);
  glEnd;

  glBegin(GL_POLYGON); // face frontal
    glNormal3f(0,0,1);
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(-10.0, 10.0, 10.0);
    glVertex3f(-10.0, -10.0, 10.0);
    glVertex3f(10.0, -10.0, 10.0);
  glEnd;

  glBegin(GL_POLYGON);  //  face lateral esquerda
    glNormal3f(-1,0,0);
    glVertex3f(-10.0, 10.0, 10.0);
    glVertex3f(-10.0, 10.0, -10.0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(-10.0, -10.0, 10.0);
  glEnd;

  glBegin(GL_POLYGON);   //  face lateral direita
    glNormal3f(1,0,0);
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(10.0, -10.0, 10.0);
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(10.0, 10.0, -10.0);
  glEnd;

  glBegin(GL_POLYGON);  //  face superior
    glNormal3f(0,1,0);
    glVertex3f(-10.0, 10.0, -10.0);
    glVertex3f(-10.0, 10.0, 10.0);
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(10.0, 10.0, -10.0);
  glEnd;

  glBegin(GL_POLYGON);   //  face inferior
    glNormal3f(0,-1,0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(10.0, -10.0, 10.0);
    glVertex3f(-10.0, -10.0, 10.0);
  glEnd;
end;

procedure Tform1.DefineLuz;
const
  LuzAmbiente : Array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1.0);
  LuzDifusa   : Array[0..3] of GLfloat = (0.4, 0.4, 0.4, 1.0);
  LuzEspecular: Array[0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
  PosicaoLuz  : Array[0..3] of GLfloat = (30.0, 20.0, 60.0, 1.0);
  Especularidade: Array[0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
begin
   // Habilita o uso de iluminao
  glEnable(GL_LIGHTING);

  // Ativa o uso da luz ambiente
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @LuzAmbiente);
  // Define os parametros da Luz nmero Zero
  glLightfv(GL_LIGHT0, GL_AMBIENT, @LuzAmbiente);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @LuzDifusa  );
  glLightfv(GL_LIGHT0, GL_SPECULAR, @LuzEspecular  );
  glLightfv(GL_LIGHT0, GL_POSITION, @PosicaoLuz );
  glEnable(GL_LIGHT0);

  // Ativa o "Color Tracking"
  glEnable(GL_COLOR_MATERIAL);

  // Define a reflectancia do material
  glMaterialfv(GL_FRONT,GL_SPECULAR, @Especularidade);

  // Define a concentrao do brilho.
  // Quanto maior o valor do Segundo parametro, mais
  // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
  glMateriali(GL_FRONT,GL_SHININESS,20);

end;

Procedure TForm1.PosicionaObservador;
begin
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   gluPerspective(AngVisao,1,1,1000);  // Deve ser colocado ANTES do gluLookAt
   gluLookAt(ObsX,ObsY,ObsZ, 0,0,29, 0,1,0);
   glMatrixMode(GL_MODELVIEW);
end;


procedure TForm1.OGLPanel1OGLPaint;
var
   a: integer;
   b: real;
begin

       PosicionaObservador();

       glLoadIdentity();  // deve-se colocar isto no incio desta rotina pois
                          // as pois as transformaes so cumulativas
       DefineLuz();

       glDisable(GL_CULL_FACE);

       glClearColor(0,0,0,0);
       glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

//
// Os trs blocos a seguir definem transformaes
// geomtricas e exibe os objetos
//
       glPushMatrix(); // Salva as transformaes geomtricas na pilha OpenGl
          glColor3f(0.0,0.0,1.0); // azul
          glRotatef(Angulo,0,0,1);  // gira ao redor do Z
          desenhaObjeto();
       glPopMatrix();  // Restaura as transformaes geomtricas salvas
                       // na pilha OpenGl

       glPushMatrix(); // Salva as transformaes geomtricas na pilha OpenGl
          glColor3f(1.0,0.0,0.0); // vermelho
          glTranslatef(10,-10,0);
          glRotatef(Angulo,0,1,0);  // gira ao redor do Y
          desenhaObjeto();
       glPopMatrix();  // Restaura as transformaes geomtricas salvas
                       // na pilha OpenGL

       glPushMatrix(); // Salva as transformaes geomtricas na pilha OpenGl
          glColor3f(0.0,1.0,0.0); // verde
          gltranslatef(-10,10,0);
          glRotatef(-Angulo,0,1,0);  // gira ao redor do Y
          desenhaObjeto();
       glPopMatrix();  // Restaura as transformaes geomtricas salvas
                       // na pilha OpenGl
       OGLPanel1.Swap;  // Conclui a exibio dos objetos OpenGL
   end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
    Angulo := Angulo + 10;
    OGLPanel1.Repaint;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    Angulo := 0;
    ObsX := 0;
    ObsY := 0;
    ObsZ := 100;
    AngVisao := 45;

end;


procedure TForm1.ScrollAngVisChange(Sender: TObject);
begin
     AngVisao:= ScrollAngVis.Position;
end;

procedure TForm1.ScrollXChange(Sender: TObject);
begin
    ObsX :=  ScrollX.Position;
    LabelObsX.Caption := 'Obs X:' + IntToStr(ScrollX.Position);
end;

procedure TForm1.ScrollYChange(Sender: TObject);
begin
    ObsY :=  ScrollY.Position;
    LabelObsY.Caption := 'Obs Y:' + IntToStr(ScrollY.Position);
end;

procedure TForm1.ScrollZChange(Sender: TObject);
begin
    ObsZ :=  ScrollZ.Position;
    LabelObsZ.Caption := 'Obs Z:' + IntToStr(ScrollZ.Position);
end;


end.
